"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.OpenId4VciHolderService = void 0;
const core_1 = require("@credo-ts/core");
const oid4vci_client_1 = require("@sphereon/oid4vci-client");
const oid4vci_common_1 = require("@sphereon/oid4vci-common");
const shared_1 = require("../shared");
const issuerMetadataUtils_1 = require("../shared/issuerMetadataUtils");
const utils_1 = require("../shared/utils");
const OpenId4VciHolderServiceOptions_1 = require("./OpenId4VciHolderServiceOptions");
let OpenId4VciHolderService = class OpenId4VciHolderService {
    constructor(logger, w3cCredentialService, jwsService) {
        this.w3cCredentialService = w3cCredentialService;
        this.jwsService = jwsService;
        this.logger = logger;
    }
    async resolveCredentialOffer(credentialOffer) {
        var _a;
        const client = await oid4vci_client_1.OpenID4VCIClient.fromURI({
            uri: credentialOffer,
            resolveOfferUri: true,
            retrieveServerMetadata: true,
            // This is a separate call, so we don't fetch it here, however it may be easier to just construct it here?
            createAuthorizationRequestURL: false,
        });
        if (!((_a = client.credentialOffer) === null || _a === void 0 ? void 0 : _a.credential_offer)) {
            throw new core_1.CredoError(`Could not resolve credential offer from '${credentialOffer}'`);
        }
        const credentialOfferPayload = client.credentialOffer.credential_offer;
        const metadata = await client.retrieveServerMetadata();
        if (!metadata.credentialIssuerMetadata) {
            throw new core_1.CredoError(`Could not retrieve issuer metadata from '${metadata.issuer}'`);
        }
        const issuerMetadata = metadata.credentialIssuerMetadata;
        this.logger.info('Fetched server metadata', {
            issuer: metadata.issuer,
            credentialEndpoint: metadata.credential_endpoint,
            tokenEndpoint: metadata.token_endpoint,
        });
        this.logger.debug('Full server metadata', metadata);
        return {
            metadata: Object.assign(Object.assign({}, metadata), { credentialIssuerMetadata: issuerMetadata }),
            credentialOfferPayload,
            offeredCredentials: (0, issuerMetadataUtils_1.getOfferedCredentials)(credentialOfferPayload.credentials, issuerMetadata.credentials_supported),
            version: client.version(),
        };
    }
    getAuthDetailsFromOfferedCredential(offeredCredential, authDetailsLocation) {
        const { format } = offeredCredential;
        const type = 'openid_credential';
        const locations = authDetailsLocation ? [authDetailsLocation] : undefined;
        if (format === shared_1.OpenId4VciCredentialFormatProfile.JwtVcJson) {
            return { type, format, types: offeredCredential.types, locations };
        }
        else if (format === shared_1.OpenId4VciCredentialFormatProfile.LdpVc ||
            format === shared_1.OpenId4VciCredentialFormatProfile.JwtVcJsonLd) {
            const credential_definition = {
                '@context': offeredCredential['@context'],
                credentialSubject: offeredCredential.credentialSubject,
                types: offeredCredential.types,
            };
            return { type, format, locations, credential_definition };
        }
        else if (format === shared_1.OpenId4VciCredentialFormatProfile.SdJwtVc) {
            return {
                type,
                format,
                locations,
                vct: offeredCredential.vct,
                claims: offeredCredential.claims,
            };
        }
        else {
            throw new core_1.CredoError(`Cannot create authorization_details. Unsupported credential format '${format}'.`);
        }
    }
    async resolveAuthorizationRequest(agentContext, resolvedCredentialOffer, authCodeFlowOptions) {
        const { credentialOfferPayload, metadata, offeredCredentials } = resolvedCredentialOffer;
        const codeVerifier = (await Promise.allSettled([agentContext.wallet.generateNonce(), agentContext.wallet.generateNonce()])).join();
        const codeVerifierSha256 = core_1.Hasher.hash(codeVerifier, 'sha-256');
        const codeChallenge = core_1.TypedArrayEncoder.toBase64URL(codeVerifierSha256);
        this.logger.debug('Converted code_verifier to code_challenge', {
            codeVerifier: codeVerifier,
            sha256: codeVerifierSha256.toString(),
            base64Url: codeChallenge,
        });
        const authDetailsLocation = metadata.credentialIssuerMetadata.authorization_server
            ? metadata.credentialIssuerMetadata.authorization_server
            : undefined;
        const authDetails = offeredCredentials
            .map((credential) => this.getAuthDetailsFromOfferedCredential(credential, authDetailsLocation))
            .filter((authDetail) => authDetail !== undefined);
        const { clientId, redirectUri, scope } = authCodeFlowOptions;
        const authorizationRequestUri = await createAuthorizationRequestUri({
            clientId,
            codeChallenge,
            redirectUri,
            credentialOffer: credentialOfferPayload,
            codeChallengeMethod: oid4vci_common_1.CodeChallengeMethod.S256,
            // TODO: Read HAIP SdJwtVc's should always be requested via scopes
            // TODO: should we now always use scopes instead of authDetails? or both????
            scope: scope !== null && scope !== void 0 ? scope : [],
            authDetails,
            metadata,
        });
        return Object.assign(Object.assign({}, authCodeFlowOptions), { codeVerifier,
            authorizationRequestUri });
    }
    async acceptCredentialOffer(agentContext, options) {
        var _a, _b, _c, _d;
        const { resolvedCredentialOffer, acceptCredentialOfferOptions, resolvedAuthorizationRequestWithCode } = options;
        const { credentialOfferPayload, metadata, version, offeredCredentials } = resolvedCredentialOffer;
        const { credentialsToRequest, userPin, credentialBindingResolver, verifyCredentialStatus } = acceptCredentialOfferOptions;
        if ((credentialsToRequest === null || credentialsToRequest === void 0 ? void 0 : credentialsToRequest.length) === 0) {
            this.logger.warn(`Accepting 0 credential offers. Returning`);
            return [];
        }
        this.logger.info(`Accepting the following credential offers '${credentialsToRequest}'`);
        const supportedJwaSignatureAlgorithms = (0, utils_1.getSupportedJwaSignatureAlgorithms)(agentContext);
        const allowedProofOfPossessionSigAlgs = acceptCredentialOfferOptions.allowedProofOfPossessionSignatureAlgorithms;
        const possibleProofOfPossessionSigAlgs = allowedProofOfPossessionSigAlgs
            ? allowedProofOfPossessionSigAlgs.filter((algorithm) => supportedJwaSignatureAlgorithms.includes(algorithm))
            : supportedJwaSignatureAlgorithms;
        if (possibleProofOfPossessionSigAlgs.length === 0) {
            throw new core_1.CredoError([
                `No possible proof of possession signature algorithm found.`,
                `Signature algorithms supported by the Agent '${supportedJwaSignatureAlgorithms.join(', ')}'`,
                `Allowed Signature algorithms '${allowedProofOfPossessionSigAlgs === null || allowedProofOfPossessionSigAlgs === void 0 ? void 0 : allowedProofOfPossessionSigAlgs.join(', ')}'`,
            ].join('\n'));
        }
        // acquire the access token
        let accessTokenResponse;
        const accessTokenClient = new oid4vci_client_1.AccessTokenClient();
        if (resolvedAuthorizationRequestWithCode) {
            const { code, codeVerifier, redirectUri } = resolvedAuthorizationRequestWithCode;
            accessTokenResponse = await accessTokenClient.acquireAccessToken({
                metadata: metadata,
                credentialOffer: { credential_offer: credentialOfferPayload },
                pin: userPin,
                code,
                codeVerifier,
                redirectUri,
            });
        }
        else {
            accessTokenResponse = await accessTokenClient.acquireAccessToken({
                metadata: metadata,
                credentialOffer: { credential_offer: credentialOfferPayload },
                pin: userPin,
            });
        }
        if (!accessTokenResponse.successBody) {
            throw new core_1.CredoError(`could not acquire access token from '${metadata.issuer}'. ${(_a = accessTokenResponse.errorBody) === null || _a === void 0 ? void 0 : _a.error}: ${(_b = accessTokenResponse.errorBody) === null || _b === void 0 ? void 0 : _b.error_description}`);
        }
        this.logger.debug('Requested OpenId4VCI Access Token.');
        const accessToken = accessTokenResponse.successBody;
        const receivedCredentials = [];
        let newCNonce;
        const credentialsSupportedToRequest = (_c = credentialsToRequest === null || credentialsToRequest === void 0 ? void 0 : credentialsToRequest.map((id) => offeredCredentials.find((credential) => credential.id === id)).filter((c, i) => {
            if (!c) {
                const offeredCredentialIds = offeredCredentials.map((c) => c.id).join(', ');
                throw new core_1.CredoError(`Credential to request '${credentialsToRequest[i]}' is not present in offered credentials. Offered credentials are ${offeredCredentialIds}`);
            }
            return true;
        })) !== null && _c !== void 0 ? _c : offeredCredentials;
        for (const offeredCredential of credentialsSupportedToRequest) {
            // Get all options for the credential request (such as which kid to use, the signature algorithm, etc)
            const { credentialBinding, signatureAlgorithm } = await this.getCredentialRequestOptions(agentContext, {
                possibleProofOfPossessionSignatureAlgorithms: possibleProofOfPossessionSigAlgs,
                offeredCredential,
                credentialBindingResolver,
            });
            // Create the proof of possession
            const proofOfPossessionBuilder = oid4vci_client_1.ProofOfPossessionBuilder.fromAccessTokenResponse({
                accessTokenResponse: accessToken,
                callbacks: { signCallback: this.proofOfPossessionSignCallback(agentContext) },
                version,
            })
                .withEndpointMetadata(metadata)
                .withAlg(signatureAlgorithm);
            if (credentialBinding.method === 'did') {
                proofOfPossessionBuilder.withClientId((0, core_1.parseDid)(credentialBinding.didUrl).did).withKid(credentialBinding.didUrl);
            }
            else if (credentialBinding.method === 'jwk') {
                proofOfPossessionBuilder.withJWK(credentialBinding.jwk.toJson());
            }
            if (newCNonce)
                proofOfPossessionBuilder.withAccessTokenNonce(newCNonce);
            const proofOfPossession = await proofOfPossessionBuilder.build();
            this.logger.debug('Generated JWS', proofOfPossession);
            // Acquire the credential
            const credentialRequestBuilder = new oid4vci_client_1.CredentialRequestClientBuilder();
            credentialRequestBuilder
                .withVersion(version)
                .withCredentialEndpoint(metadata.credential_endpoint)
                .withTokenFromResponse(accessToken);
            const credentialRequestClient = credentialRequestBuilder.build();
            const credentialResponse = await credentialRequestClient.acquireCredentialsUsingProof({
                proofInput: proofOfPossession,
                credentialTypes: (0, issuerMetadataUtils_1.getTypesFromCredentialSupported)(offeredCredential),
                format: offeredCredential.format,
            });
            newCNonce = (_d = credentialResponse.successBody) === null || _d === void 0 ? void 0 : _d.c_nonce;
            // Create credential, but we don't store it yet (only after the user has accepted the credential)
            const credential = await this.handleCredentialResponse(agentContext, credentialResponse, {
                verifyCredentialStatus: verifyCredentialStatus !== null && verifyCredentialStatus !== void 0 ? verifyCredentialStatus : false,
            });
            this.logger.debug('Full credential', credential);
            receivedCredentials.push(credential);
        }
        return receivedCredentials;
    }
    /**
     * Get the options for the credential request. Internally this will resolve the proof of possession
     * requirements, and based on that it will call the proofOfPossessionVerificationMethodResolver to
     * allow the caller to select the correct verification method based on the requirements for the proof
     * of possession.
     */
    async getCredentialRequestOptions(agentContext, options) {
        const { signatureAlgorithm, supportedDidMethods, supportsAllDidMethods, supportsJwk } = this.getProofOfPossessionRequirements(agentContext, {
            credentialToRequest: options.offeredCredential,
            possibleProofOfPossessionSignatureAlgorithms: options.possibleProofOfPossessionSignatureAlgorithms,
        });
        const JwkClass = (0, core_1.getJwkClassFromJwaSignatureAlgorithm)(signatureAlgorithm);
        if (!JwkClass) {
            throw new core_1.CredoError(`Could not determine JWK key type of the JWA signature algorithm '${signatureAlgorithm}'`);
        }
        const supportedVerificationMethods = (0, core_1.getSupportedVerificationMethodTypesFromKeyType)(JwkClass.keyType);
        const format = options.offeredCredential.format;
        // Now we need to determine how the credential will be bound to us
        const credentialBinding = await options.credentialBindingResolver({
            credentialFormat: format,
            signatureAlgorithm,
            supportedVerificationMethods,
            keyType: JwkClass.keyType,
            supportedCredentialId: options.offeredCredential.id,
            supportsAllDidMethods,
            supportedDidMethods,
            supportsJwk,
        });
        // Make sure the issuer of proof of possession is valid according to openid issuer metadata
        if (credentialBinding.method === 'did' &&
            !supportsAllDidMethods &&
            // If supportedDidMethods is undefined, it means the issuer didn't include the binding methods in the metadata
            // The user can still select a verification method, but we can't validate it
            supportedDidMethods !== undefined &&
            !supportedDidMethods.find((supportedDidMethod) => credentialBinding.didUrl.startsWith(supportedDidMethod))) {
            const { method } = (0, core_1.parseDid)(credentialBinding.didUrl);
            const supportedDidMethodsString = supportedDidMethods.join(', ');
            throw new core_1.CredoError(`Resolved credential binding for proof of possession uses did method '${method}', but issuer only supports '${supportedDidMethodsString}'`);
        }
        else if (credentialBinding.method === 'jwk' && !supportsJwk) {
            throw new core_1.CredoError(`Resolved credential binding for proof of possession uses jwk, but openid issuer does not support 'jwk' cryptographic binding method`);
        }
        // FIXME: we don't have the verification method here
        // Make sure the verification method uses a supported verification method type
        // if (!supportedVerificationMethods.includes(verificationMethod.type)) {
        //   const supportedVerificationMethodsString = supportedVerificationMethods.join(', ')
        //   throw new CredoError(
        //     `Verification method uses verification method type '${verificationMethod.type}', but only '${supportedVerificationMethodsString}' verification methods are supported for key type '${JwkClass.keyType}'`
        //   )
        // }
        return { credentialBinding, signatureAlgorithm };
    }
    /**
     * Get the requirements for creating the proof of possession. Based on the allowed
     * credential formats, the allowed proof of possession signature algorithms, and the
     * credential type, this method will select the best credential format and signature
     * algorithm to use, based on the order of preference.
     */
    getProofOfPossessionRequirements(agentContext, options) {
        var _a, _b;
        const { credentialToRequest } = options;
        if (!OpenId4VciHolderServiceOptions_1.openId4VciSupportedCredentialFormats.includes(credentialToRequest.format)) {
            throw new core_1.CredoError([
                `Requested credential with format '${credentialToRequest.format}',`,
                `for the credential with id '${credentialToRequest.id},`,
                `but the wallet only supports the following formats '${OpenId4VciHolderServiceOptions_1.openId4VciSupportedCredentialFormats.join(', ')}'`,
            ].join('\n'));
        }
        // For each of the supported algs, find the key types, then find the proof types
        const signatureSuiteRegistry = agentContext.dependencyManager.resolve(core_1.SignatureSuiteRegistry);
        let signatureAlgorithm;
        const issuerSupportedCryptographicSuites = credentialToRequest.cryptographic_suites_supported;
        const issuerSupportedBindingMethods = credentialToRequest.cryptographic_binding_methods_supported;
        // If undefined, it means the issuer didn't include the cryptographic suites in the metadata
        // We just guess that the first one is supported
        if (issuerSupportedCryptographicSuites === undefined) {
            signatureAlgorithm = options.possibleProofOfPossessionSignatureAlgorithms[0];
        }
        else {
            switch (credentialToRequest.format) {
                case shared_1.OpenId4VciCredentialFormatProfile.JwtVcJson:
                case shared_1.OpenId4VciCredentialFormatProfile.JwtVcJsonLd:
                case shared_1.OpenId4VciCredentialFormatProfile.SdJwtVc:
                    signatureAlgorithm = options.possibleProofOfPossessionSignatureAlgorithms.find((signatureAlgorithm) => issuerSupportedCryptographicSuites.includes(signatureAlgorithm));
                    break;
                case shared_1.OpenId4VciCredentialFormatProfile.LdpVc:
                    signatureAlgorithm = options.possibleProofOfPossessionSignatureAlgorithms.find((signatureAlgorithm) => {
                        const JwkClass = (0, core_1.getJwkClassFromJwaSignatureAlgorithm)(signatureAlgorithm);
                        if (!JwkClass)
                            return false;
                        const matchingSuite = signatureSuiteRegistry.getAllByKeyType(JwkClass.keyType);
                        if (matchingSuite.length === 0)
                            return false;
                        return issuerSupportedCryptographicSuites.includes(matchingSuite[0].proofType);
                    });
                    break;
                default:
                    throw new core_1.CredoError(`Unsupported credential format.`);
            }
        }
        if (!signatureAlgorithm) {
            throw new core_1.CredoError(`Could not establish signature algorithm for format ${credentialToRequest.format} and id ${credentialToRequest.id}`);
        }
        const supportsAllDidMethods = (_a = issuerSupportedBindingMethods === null || issuerSupportedBindingMethods === void 0 ? void 0 : issuerSupportedBindingMethods.includes('did')) !== null && _a !== void 0 ? _a : false;
        const supportedDidMethods = issuerSupportedBindingMethods === null || issuerSupportedBindingMethods === void 0 ? void 0 : issuerSupportedBindingMethods.filter((method) => method.startsWith('did:'));
        const supportsJwk = (_b = issuerSupportedBindingMethods === null || issuerSupportedBindingMethods === void 0 ? void 0 : issuerSupportedBindingMethods.includes('jwk')) !== null && _b !== void 0 ? _b : false;
        return {
            signatureAlgorithm,
            supportedDidMethods,
            supportsAllDidMethods,
            supportsJwk,
        };
    }
    async handleCredentialResponse(agentContext, credentialResponse, options) {
        var _a, _b, _c, _d, _e, _f;
        const { verifyCredentialStatus } = options;
        this.logger.debug('Credential request response', credentialResponse);
        if (!credentialResponse.successBody || !credentialResponse.successBody.credential) {
            throw new core_1.CredoError(`Did not receive a successful credential response. ${(_a = credentialResponse.errorBody) === null || _a === void 0 ? void 0 : _a.error}: ${(_b = credentialResponse.errorBody) === null || _b === void 0 ? void 0 : _b.error_description}`);
        }
        const format = credentialResponse.successBody.format;
        if (format === shared_1.OpenId4VciCredentialFormatProfile.SdJwtVc) {
            if (typeof credentialResponse.successBody.credential !== 'string')
                throw new core_1.CredoError(`Received a credential of format ${shared_1.OpenId4VciCredentialFormatProfile.SdJwtVc}, but the credential is not a string. ${JSON.stringify(credentialResponse.successBody.credential)}`);
            const sdJwtVcApi = agentContext.dependencyManager.resolve(core_1.SdJwtVcApi);
            const { verification, sdJwtVc } = await sdJwtVcApi.verify({
                compactSdJwtVc: credentialResponse.successBody.credential,
            });
            if (!verification.isValid) {
                agentContext.config.logger.error('Failed to validate credential', { verification });
                throw new core_1.CredoError(`Failed to validate sd-jwt-vc credential. Results = ${JSON.stringify(verification)}`);
            }
            return sdJwtVc;
        }
        else if (format === shared_1.OpenId4VciCredentialFormatProfile.JwtVcJson ||
            format === shared_1.OpenId4VciCredentialFormatProfile.JwtVcJsonLd) {
            const credential = core_1.W3cJwtVerifiableCredential.fromSerializedJwt(credentialResponse.successBody.credential);
            const result = await this.w3cCredentialService.verifyCredential(agentContext, {
                credential,
                verifyCredentialStatus,
            });
            if (!result.isValid) {
                agentContext.config.logger.error('Failed to validate credential', { result });
                throw new core_1.CredoError(`Failed to validate credential, error = ${(_d = (_c = result.error) === null || _c === void 0 ? void 0 : _c.message) !== null && _d !== void 0 ? _d : 'Unknown'}`);
            }
            return credential;
        }
        else if (format === shared_1.OpenId4VciCredentialFormatProfile.LdpVc) {
            const credential = core_1.W3cJsonLdVerifiableCredential.fromJson(credentialResponse.successBody.credential);
            const result = await this.w3cCredentialService.verifyCredential(agentContext, {
                credential,
                verifyCredentialStatus,
            });
            if (!result.isValid) {
                agentContext.config.logger.error('Failed to validate credential', { result });
                throw new core_1.CredoError(`Failed to validate credential, error = ${(_f = (_e = result.error) === null || _e === void 0 ? void 0 : _e.message) !== null && _f !== void 0 ? _f : 'Unknown'}`);
            }
            return credential;
        }
        throw new core_1.CredoError(`Unsupported credential format ${credentialResponse.successBody.format}`);
    }
    proofOfPossessionSignCallback(agentContext) {
        return async (jwt, kid) => {
            if (!jwt.header)
                throw new core_1.CredoError('No header present on JWT');
            if (!jwt.payload)
                throw new core_1.CredoError('No payload present on JWT');
            if (kid && jwt.header.jwk) {
                throw new core_1.CredoError('Both KID and JWK are present in the callback. Only one can be present');
            }
            let key;
            if (kid) {
                if (!kid.startsWith('did:')) {
                    throw new core_1.CredoError(`kid '${kid}' is not a DID. Only dids are supported for kid`);
                }
                else if (!kid.includes('#')) {
                    throw new core_1.CredoError(`kid '${kid}' does not contain a fragment. kid MUST point to a specific key in the did document.`);
                }
                const didsApi = agentContext.dependencyManager.resolve(core_1.DidsApi);
                const didDocument = await didsApi.resolveDidDocument(kid);
                const verificationMethod = didDocument.dereferenceKey(kid, ['authentication']);
                key = (0, core_1.getKeyFromVerificationMethod)(verificationMethod);
            }
            else if (jwt.header.jwk) {
                key = (0, core_1.getJwkFromJson)(jwt.header.jwk).key;
            }
            else {
                throw new core_1.CredoError('No KID or JWK is present in the callback');
            }
            const jwk = (0, core_1.getJwkFromKey)(key);
            if (!jwk.supportsSignatureAlgorithm(jwt.header.alg)) {
                throw new core_1.CredoError(`key type '${jwk.keyType}', does not support the JWS signature alg '${jwt.header.alg}'`);
            }
            // We don't support these properties, remove them, so we can pass all other header properties to the JWS service
            if (jwt.header.x5c)
                throw new core_1.CredoError('x5c is not supported');
            // eslint-disable-next-line @typescript-eslint/no-unused-vars
            const _a = jwt.header, { x5c: _x5c } = _a, supportedHeaderOptions = __rest(_a, ["x5c"]);
            const jws = await this.jwsService.createJwsCompact(agentContext, {
                key,
                payload: core_1.JsonEncoder.toBuffer(jwt.payload),
                protectedHeaderOptions: Object.assign(Object.assign({}, supportedHeaderOptions), { 
                    // only pass jwk if it was present in the header
                    jwk: jwt.header.jwk ? jwk : undefined }),
            });
            return jws;
        };
    }
};
OpenId4VciHolderService = __decorate([
    (0, core_1.injectable)(),
    __param(0, (0, core_1.inject)(core_1.InjectionSymbols.Logger)),
    __metadata("design:paramtypes", [Object, core_1.W3cCredentialService,
        core_1.JwsService])
], OpenId4VciHolderService);
exports.OpenId4VciHolderService = OpenId4VciHolderService;
// NOTE: this is also defined in the sphereon lib, but we use
// this custom method to get PAR working and because we don't
// use the oid4vci client in sphereon's lib
// Once PAR is supported in the sphereon lib, we should to try remove this
// and use the one from the sphereon lib
async function createAuthorizationRequestUri(options) {
    var _a, _b, _c;
    const { scope, authDetails, metadata, clientId, codeChallenge, codeChallengeMethod, redirectUri } = options;
    let nonEmptyScope = !scope || scope.length === 0 ? undefined : scope;
    const nonEmptyAuthDetails = !authDetails || authDetails.length === 0 ? undefined : authDetails;
    // Scope and authorization_details can be used in the same authorization request
    // https://datatracker.ietf.org/doc/html/draft-ietf-oauth-rar-23#name-relationship-to-scope-param
    if (!nonEmptyScope && !nonEmptyAuthDetails) {
        throw new core_1.CredoError(`Please provide a 'scope' or 'authDetails' via the options.`);
    }
    // Authorization servers supporting PAR SHOULD include the URL of their pushed authorization request endpoint in their authorization server metadata document
    // Note that the presence of pushed_authorization_request_endpoint is sufficient for a client to determine that it may use the PAR flow.
    const parEndpoint = metadata.credentialIssuerMetadata.pushed_authorization_request_endpoint;
    const authorizationEndpoint = (_a = metadata.credentialIssuerMetadata) === null || _a === void 0 ? void 0 : _a.authorization_endpoint;
    if (!authorizationEndpoint && !parEndpoint) {
        throw new core_1.CredoError("Server metadata does not contain an 'authorization_endpoint' which is required for the 'Authorization Code Flow'");
    }
    // add 'openid' scope if not present
    if (nonEmptyScope && !(nonEmptyScope === null || nonEmptyScope === void 0 ? void 0 : nonEmptyScope.includes('openid'))) {
        nonEmptyScope = ['openid', ...nonEmptyScope];
    }
    const queryObj = {
        client_id: clientId,
        response_type: oid4vci_common_1.ResponseType.AUTH_CODE,
        code_challenge_method: codeChallengeMethod,
        code_challenge: codeChallenge,
        redirect_uri: redirectUri,
    };
    if (nonEmptyScope)
        queryObj['scope'] = nonEmptyScope.join(' ');
    if (nonEmptyAuthDetails)
        queryObj['authorization_details'] = JSON.stringify((0, issuerMetadataUtils_1.handleAuthorizationDetails)(nonEmptyAuthDetails, metadata));
    const issuerState = (_c = (_b = options.credentialOffer.grants) === null || _b === void 0 ? void 0 : _b.authorization_code) === null || _c === void 0 ? void 0 : _c.issuer_state;
    if (issuerState)
        queryObj['issuer_state'] = issuerState;
    if (parEndpoint) {
        const body = new URLSearchParams(queryObj);
        const response = await (0, oid4vci_client_1.formPost)(parEndpoint, body);
        if (!response.successBody) {
            throw new core_1.CredoError(`Could not acquire the authorization request uri from '${parEndpoint}'`);
        }
        return (0, oid4vci_common_1.convertJsonToURI)({ request_uri: response.successBody.request_uri, client_id: clientId, response_type: oid4vci_common_1.ResponseType.AUTH_CODE }, {
            baseUrl: authorizationEndpoint,
            uriTypeProperties: ['request_uri', 'client_id', 'response_type'],
            mode: oid4vci_common_1.JsonURIMode.X_FORM_WWW_URLENCODED,
        });
    }
    else {
        return (0, oid4vci_common_1.convertJsonToURI)(queryObj, {
            baseUrl: authorizationEndpoint,
            uriTypeProperties: ['redirect_uri', 'scope', 'authorization_details', 'issuer_state'],
            mode: oid4vci_common_1.JsonURIMode.X_FORM_WWW_URLENCODED,
        });
    }
}
//# sourceMappingURL=OpenId4VciHolderService.js.map