"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.verifyTokenRequest = exports.handleTokenRequest = exports.configureAccessTokenEndpoint = void 0;
const core_1 = require("@credo-ts/core");
const oid4vci_common_1 = require("@sphereon/oid4vci-common");
const oid4vci_issuer_1 = require("@sphereon/oid4vci-issuer");
const router_1 = require("../../shared/router");
const OpenId4VcIssuerService_1 = require("../OpenId4VcIssuerService");
const OpenId4VcCNonceStateManager_1 = require("../repository/OpenId4VcCNonceStateManager");
const OpenId4VcCredentialOfferSessionStateManager_1 = require("../repository/OpenId4VcCredentialOfferSessionStateManager");
function configureAccessTokenEndpoint(router, config) {
    router.post(config.endpointPath, verifyTokenRequest({ preAuthorizedCodeExpirationInSeconds: config.preAuthorizedCodeExpirationInSeconds }), handleTokenRequest(config));
}
exports.configureAccessTokenEndpoint = configureAccessTokenEndpoint;
function getJwtSignerCallback(agentContext, signerPublicKey, config) {
    return async (jwt, _kid) => {
        var _a;
        if (_kid) {
            throw new core_1.CredoError('Kid should not be supplied externally.');
        }
        if (jwt.header.kid || jwt.header.jwk) {
            throw new core_1.CredoError('kid or jwk should not be present in access token header before signing');
        }
        const jwsService = agentContext.dependencyManager.resolve(core_1.JwsService);
        const alg = (_a = (0, core_1.getJwkClassFromKeyType)(signerPublicKey.keyType)) === null || _a === void 0 ? void 0 : _a.supportedSignatureAlgorithms[0];
        if (!alg) {
            throw new core_1.CredoError(`No supported signature algorithms for key type: ${signerPublicKey.keyType}`);
        }
        // FIXME: the iat and exp implementation in OID4VCI is incorrect so we override the values here
        // https://github.com/Sphereon-Opensource/OID4VCI/pull/99
        // https://github.com/Sphereon-Opensource/OID4VCI/pull/101
        const iat = Math.floor(new Date().getTime() / 1000);
        jwt.payload.iat = iat;
        jwt.payload.exp = iat + config.tokenExpiresInSeconds;
        const jwk = (0, core_1.getJwkFromKey)(signerPublicKey);
        const signedJwt = await jwsService.createJwsCompact(agentContext, {
            protectedHeaderOptions: Object.assign(Object.assign({}, jwt.header), { jwk, alg }),
            payload: new core_1.JwtPayload(jwt.payload),
            key: signerPublicKey,
        });
        return signedJwt;
    };
}
function handleTokenRequest(config) {
    const { tokenExpiresInSeconds, cNonceExpiresInSeconds } = config;
    return async (request, response, next) => {
        response.set({ 'Cache-Control': 'no-store', Pragma: 'no-cache' });
        const requestContext = (0, router_1.getRequestContext)(request);
        const { agentContext, issuer } = requestContext;
        if (request.body.grant_type !== oid4vci_common_1.GrantTypes.PRE_AUTHORIZED_CODE) {
            return response.status(400).json({
                error: oid4vci_common_1.TokenErrorResponse.invalid_request,
                error_description: oid4vci_common_1.PRE_AUTHORIZED_CODE_REQUIRED_ERROR,
            });
        }
        const openId4VcIssuerService = agentContext.dependencyManager.resolve(OpenId4VcIssuerService_1.OpenId4VcIssuerService);
        const issuerMetadata = openId4VcIssuerService.getIssuerMetadata(agentContext, issuer);
        const accessTokenSigningKey = core_1.Key.fromFingerprint(issuer.accessTokenPublicKeyFingerprint);
        try {
            const accessTokenResponse = await (0, oid4vci_issuer_1.createAccessTokenResponse)(request.body, {
                credentialOfferSessions: new OpenId4VcCredentialOfferSessionStateManager_1.OpenId4VcCredentialOfferSessionStateManager(agentContext, issuer.issuerId),
                tokenExpiresIn: tokenExpiresInSeconds * 1000,
                accessTokenIssuer: issuerMetadata.issuerUrl,
                cNonce: await agentContext.wallet.generateNonce(),
                cNonceExpiresIn: cNonceExpiresInSeconds,
                cNonces: new OpenId4VcCNonceStateManager_1.OpenId4VcCNonceStateManager(agentContext, issuer.issuerId),
                accessTokenSignerCallback: getJwtSignerCallback(agentContext, accessTokenSigningKey, config),
            });
            response.status(200).json(accessTokenResponse);
        }
        catch (error) {
            (0, router_1.sendErrorResponse)(response, agentContext.config.logger, 400, oid4vci_common_1.TokenErrorResponse.invalid_request, error);
        }
        // NOTE: if we don't call next, the agentContext session handler will NOT be called
        next();
    };
}
exports.handleTokenRequest = handleTokenRequest;
function verifyTokenRequest(options) {
    return async (request, response, next) => {
        const { agentContext, issuer } = (0, router_1.getRequestContext)(request);
        try {
            await (0, oid4vci_issuer_1.assertValidAccessTokenRequest)(request.body, {
                // we use seconds instead of milliseconds for consistency
                expirationDuration: options.preAuthorizedCodeExpirationInSeconds * 1000,
                credentialOfferSessions: new OpenId4VcCredentialOfferSessionStateManager_1.OpenId4VcCredentialOfferSessionStateManager(agentContext, issuer.issuerId),
            });
        }
        catch (error) {
            if (error instanceof oid4vci_common_1.TokenError) {
                (0, router_1.sendErrorResponse)(response, agentContext.config.logger, error.statusCode, error.responseError + error.getDescription(), error);
            }
            else {
                (0, router_1.sendErrorResponse)(response, agentContext.config.logger, 400, oid4vci_common_1.TokenErrorResponse.invalid_request, error);
            }
        }
        // NOTE: if we don't call next, the agentContext session handler will NOT be called
        next();
    };
}
exports.verifyTokenRequest = verifyTokenRequest;
//# sourceMappingURL=accessTokenEndpoint.js.map