"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.handleAuthorizationDetails = exports.getOfferedCredentials = exports.getTypesFromCredentialSupported = void 0;
const core_1 = require("@credo-ts/core");
/**
 * Get all `types` from a `CredentialSupported` object.
 *
 * Depending on the format, the types may be nested, or have different a different name/type
 */
function getTypesFromCredentialSupported(credentialSupported) {
    if (credentialSupported.format === 'jwt_vc_json-ld' ||
        credentialSupported.format === 'ldp_vc' ||
        credentialSupported.format === 'jwt_vc_json' ||
        credentialSupported.format === 'jwt_vc') {
        return credentialSupported.types;
    }
    else if (credentialSupported.format === 'vc+sd-jwt') {
        return [credentialSupported.vct];
    }
    throw Error(`Unable to extract types from credentials supported. Unknown format ${credentialSupported.format}`);
}
exports.getTypesFromCredentialSupported = getTypesFromCredentialSupported;
/**
 * Returns all entries from the credential offer with the associated metadata resolved. For 'id' entries, the associated `credentials_supported` object is resolved from the issuer metadata.
 * For inline entries, an error is thrown.
 */
function getOfferedCredentials(offeredCredentials, allCredentialsSupported) {
    const credentialsSupported = [];
    for (const offeredCredential of offeredCredentials) {
        // In draft 12 inline credential offers are removed. It's easier to already remove support now.
        if (typeof offeredCredential !== 'string') {
            throw new core_1.CredoError('Only referenced credentials pointing to an id in credentials_supported issuer metadata are supported');
        }
        const foundSupportedCredential = allCredentialsSupported.find((supportedCredential) => supportedCredential.id !== undefined && supportedCredential.id === offeredCredential);
        // Make sure the issuer metadata includes the offered credential.
        if (!foundSupportedCredential) {
            throw new Error(`Offered credential '${offeredCredential}' is not part of credentials_supported of the issuer metadata.`);
        }
        credentialsSupported.push(foundSupportedCredential);
    }
    return credentialsSupported;
}
exports.getOfferedCredentials = getOfferedCredentials;
// copied from sphereon as the method is only available on the client
function handleAuthorizationDetails(authorizationDetails, metadata) {
    if (Array.isArray(authorizationDetails)) {
        return authorizationDetails.map((value) => handleLocations(value, metadata));
    }
    else {
        return handleLocations(authorizationDetails, metadata);
    }
}
exports.handleAuthorizationDetails = handleAuthorizationDetails;
// copied from sphereon as the method is only available on the client
function handleLocations(authorizationDetails, metadata) {
    var _a;
    if (typeof authorizationDetails === 'string')
        return authorizationDetails;
    if (((_a = metadata.credentialIssuerMetadata) === null || _a === void 0 ? void 0 : _a.authorization_server) || metadata.authorization_endpoint) {
        if (!authorizationDetails.locations)
            authorizationDetails.locations = [metadata.issuer];
        else if (Array.isArray(authorizationDetails.locations))
            authorizationDetails.locations.push(metadata.issuer);
        else
            authorizationDetails.locations = [authorizationDetails.locations, metadata.issuer];
    }
    return authorizationDetails;
}
//# sourceMappingURL=issuerMetadataUtils.js.map